@@ -93,16 +93,9 @@ def generate_hgnc_terms():
 
 
 def generate_chebi_terms():
-    fname = os.path.join(indra_resources, 'chebi_entries.tsv')
-    logger.info('Loading %s' % fname)
-    terms = []
-    for row in read_csv(fname, header=True, delimiter='\t'):
-        db = 'CHEBI'
-        id = 'CHEBI:' + row['CHEBI_ID']
-        name = row['NAME']
-        term = Term(normalize(name), name, db, id, name, 'name', 'chebi')
-        terms.append(term)
-    logger.info('Loaded %d terms' % len(terms))
+    # We can get standard names directly from the OBO
+    terms = _generate_obo_terms('chebi', ignore_mappings=True,
+                                map_to_ns={})
 
     # Now we add synonyms
     # NOTE: this file is not in version control. The file is available
@@ -130,7 +123,6 @@ def generate_chebi_terms():
                         row['COMPOUND_ID'])
             continue
         db = 'CHEBI'
-        id = 'CHEBI:%s' % chebi_id
         name = str(row['NAME'])
         chebi_name = \
             chebi_client.get_chebi_name_from_id(chebi_id, offline=True)
@@ -143,7 +135,7 @@ def generate_chebi_terms():
         if is_aa_sequence(chebi_name) and re.match(r'(^[A-Z-]+$)', name):
             continue
 
-        term_args = (normalize(name), name, db, id, chebi_name, 'synonym',
+        term_args = (normalize(name), name, db, chebi_id, chebi_name, 'synonym',
                      'chebi')
         if term_args in added:
             continue
@@ -166,54 +158,60 @@ aa_abbrevs = {aa['short_name'].capitalize() for aa in amino_acids.values()}
 
 
 def generate_mesh_terms(ignore_mappings=False):
-    # Load MeSH HGNC/FPLX mappings
-    mesh_names_file = os.path.join(indra_resources,
-                                   'mesh_id_label_mappings.tsv')
+    mesh_name_files = ['mesh_id_label_mappings.tsv',
+                       'mesh_supp_id_label_mappings.tsv']
     terms = []
-    for row in read_csv(mesh_names_file, header=False, delimiter='\t'):
-        db_id = row[0]
-        text_name = row[1]
-        mapping = mesh_mappings.get(db_id)
-        if not ignore_mappings and mapping and mapping[0] \
-                not in {'EFO', 'HP', 'DOID'}:
-            db, db_id, name = mapping
-            status = 'synonym'
-        else:
-            db = 'MESH'
-            status = 'name'
-            name = text_name
-        term = Term(normalize(text_name), text_name, db, db_id, name,
-                    status, 'mesh')
-        terms.append(term)
-        synonyms = row[2]
-        if row[2]:
-            synonyms = synonyms.split('|')
-            for synonym in synonyms:
-                term = Term(normalize(synonym), synonym, db, db_id, name,
-                            'synonym', 'mesh')
-                terms.append(term)
-    logger.info('Loaded %d terms' % len(terms))
+    for fname in mesh_name_files:
+        mesh_names_file = os.path.join(indra_resources, fname)
+        for row in read_csv(mesh_names_file, header=False, delimiter='\t'):
+            db_id = row[0]
+            text_name = row[1]
+            mapping = mesh_mappings.get(db_id)
+            if not ignore_mappings and mapping and mapping[0] \
+                    not in {'EFO', 'HP', 'DOID'}:
+                db, db_id, name = mapping
+                status = 'synonym'
+            else:
+                db = 'MESH'
+                status = 'name'
+                name = text_name
+            term = Term(normalize(text_name), text_name, db, db_id, name,
+                        status, 'mesh')
+            terms.append(term)
+            synonyms = row[2]
+            if row[2]:
+                synonyms = synonyms.split('|')
+                for synonym in synonyms:
+                    term = Term(normalize(synonym), synonym, db, db_id, name,
+                                'synonym', 'mesh')
+                    terms.append(term)
+        logger.info('Loaded %d terms' % len(terms))
     return terms
 
 
 def generate_go_terms():
-    # TODO: add synonyms for GO terms here
-    fname = os.path.join(indra_resources, 'go_id_label_mappings.tsv')
+    fname = os.path.join(indra_resources, 'go.json')
     logger.info('Loading %s' % fname)
+    with open(fname, 'r') as fh:
+        entries = json.load(fh)
     terms = []
-    for row in read_csv(fname, delimiter='\t'):
-        if not row[0].startswith('GO'):
-            continue
-        if 'obsolete' in row[1]:
-            continue
-        term = Term(normalize(row[1]), row[1], 'GO', row[0], row[1],
-                    'name', 'go')
+    for entry in entries:
+        go_id = entry['id']
+        name = entry['name']
+        # First handle the name term
+        term = Term(normalize(name), name, 'GO', go_id, name, 'name', 'go')
         terms.append(term)
+        # Next look at synonyms, sometimes those match the name so we
+        # deduplicate
+        for synonym in set(entry.get('synonyms', [])) - {name}:
+            term = Term(normalize(synonym), synonym, 'GO', go_id, name,
+                        'synonym', 'go')
+            terms.append(term)
     logger.info('Loaded %d terms' % len(terms))
     return terms
 
 
-def generate_famplex_terms():
+def generate_famplex_terms(ignore_mappings=False):
     fname = os.path.join(indra_resources, 'famplex', 'grounding_map.csv')
     logger.info('Loading %s' % fname)
     terms = []
@@ -254,7 +252,8 @@ def generate_famplex_terms():
         elif 'MESH' in groundings:
             id = groundings['MESH']
             mesh_mapping = mesh_mappings.get(id)
-            db, db_id, name = mesh_mapping if mesh_mapping else \
+            db, db_id, name = mesh_mapping if (mesh_mapping
+                                               and not ignore_mappings) else \
                 ('MESH', id, mesh_client.get_mesh_name(id))
             term = Term(norm_txt, txt, db, db_id, name, 'assertion', 'famplex')
         else:
@@ -369,101 +368,114 @@ def generate_adeft_terms():
     return terms
 
 
-def generate_doid_terms():
-    return _generate_obo_terms('doid')
+def generate_doid_terms(ignore_mappings=False):
+    return _generate_obo_terms('doid', ignore_mappings)
 
 
-def generate_efo_terms():
-    return _generate_obo_terms('efo')
+def generate_efo_terms(ignore_mappings=False):
+    return _generate_obo_terms('efo', ignore_mappings)
 
 
-def generate_hp_terms():
-    return _generate_obo_terms('hp')
+def generate_hp_terms(ignore_mappings=False):
+    return _generate_obo_terms('hp', ignore_mappings)
 
 
-def _generate_obo_terms(prefix):
+def terms_from_obo_json_entry(entry, prefix, ignore_mappings=False,
+                              map_to_ns=None):
+    if map_to_ns is None:
+        map_to_ns = {'MESH', 'DOID'}
+    terms = []
+    db, db_id, name = prefix.upper(), entry['id'], entry['name']
+    # We first need to decide if we prioritize another name space
+    xref_dict = {xr['namespace']: xr['id'] for xr in entry.get('xrefs', [])}
+    # Handle MeSH mappings first
+    auto_mesh_mapping = mesh_mappings_reverse.get((db, db_id))
+    if auto_mesh_mapping and not ignore_mappings:
+        db, db_id, name = ('MESH', auto_mesh_mapping[0],
+                           auto_mesh_mapping[1])
+    elif 'MESH' in map_to_ns and ('MESH' in xref_dict or 'MSH' in xref_dict):
+        mesh_id = xref_dict.get('MESH') or xref_dict.get('MSH')
+        # Since we currently only include regular MeSH terms (which start
+        # with D), we only need to do the mapping if that's the case.
+        # We don't map any supplementary terms that start with C.
+        if mesh_id.startswith('D'):
+            mesh_name = mesh_client.get_mesh_name(mesh_id)
+            if mesh_name:
+                # Here we need to check if we further map the MeSH ID to
+                # another namespace
+                mesh_mapping = mesh_mappings.get(mesh_id)
+                db, db_id, name = mesh_mapping if \
+                    (mesh_mapping and (mesh_mapping[0]
+                                       not in {'EFO', 'HP', 'DOID'})) \
+                    else ('MESH', mesh_id, mesh_name)
+    # Next we look at mappings to DOID
+    # TODO: are we sure that the DOIDs that we get here (from e.g., EFO)
+    # cannot be mapped further to MeSH per the DOID resource file?
+    elif 'DOID' in map_to_ns and 'DOID' in xref_dict:
+        doid = xref_dict['DOID']
+        if not doid.startswith('DOID:'):
+            doid = 'DOID:' + doid
+        doid_prim_id = doid_client.get_doid_id_from_doid_alt_id(doid)
+        if doid_prim_id:
+            doid = doid_prim_id
+        doid_name = doid_client.get_doid_name_from_doid_id(doid)
+        # If we don't get a name here, it's likely because an entry is
+        # obsolete so we don't do the mapping
+        if doid_name:
+            db, db_id, name = 'DOID', doid, doid_name
+
+    # Add a term for the name first
+    name_term = Term(
+        norm_text=normalize(name),
+        text=name,
+        db=db,
+        id=db_id,
+        entry_name=name,
+        status='name',
+        source=prefix,
+    )
+    terms.append(name_term)
+
+    # Then add all the synonyms
+    for synonym in set(entry.get('synonyms', [])):
+        # Some synonyms are tagged as ambiguous, we remove these
+        if 'ambiguous' in synonym.lower():
+            continue
+        # Some synonyms contain a "formerly" clause, we remove these
+        match = re.match(r'(.+) \(formerly', synonym)
+        if match:
+            synonym = match.groups()[0]
+        # Some synonyms contain additional annotations
+        # e.g. Hyperplasia of facial adipose tissue" NARROW
+        # [ORCID:0000-0001-5889-4463]
+        # If this is the case, we strip these off
+        match = re.match(r'([^"]+)', synonym)
+        if match:
+            synonym = match.groups()[0]
+
+        synonym_term = Term(
+            norm_text=normalize(synonym),
+            text=synonym,
+            db=db,
+            id=db_id,
+            entry_name=name,
+            status='synonym',
+            source=prefix,
+        )
+        terms.append(synonym_term)
+    return terms
+
+
+def _generate_obo_terms(prefix, ignore_mappings=False, map_to_ns=None):
     filename = os.path.join(indra_resources, '%s.json' % prefix)
     logger.info('Loading %s', filename)
     with open(filename) as file:
         entries = json.load(file)
-
     terms = []
     for entry in entries:
-        db, db_id, name = prefix.upper(), entry['id'], entry['name']
-        # We first need to decide if we prioritize another name space
-        xref_dict = {xr['namespace']: xr['id'] for xr in entry['xrefs']}
-        # Handle MeSH mappings first
-        auto_mesh_mapping = mesh_mappings_reverse.get((db, db_id))
-        if auto_mesh_mapping:
-            db, db_id, name = ('MESH', auto_mesh_mapping[0],
-                               auto_mesh_mapping[1])
-        elif 'MESH' in xref_dict or 'MSH' in xref_dict:
-            mesh_id = xref_dict.get('MESH') or xref_dict.get('MSH')
-            # Since we currently only include regular MeSH terms (which start
-            # with D), we only need to do the mapping if that's the case.
-            # We don't map any supplementary terms that start with C.
-            if mesh_id.startswith('D'):
-                mesh_name = mesh_client.get_mesh_name(mesh_id)
-                if mesh_name:
-                    # Here we need to check if we further map the MeSH ID to
-                    # another namespace
-                    mesh_mapping = mesh_mappings.get(mesh_id)
-                    db, db_id, name = mesh_mapping if (mesh_mapping and \
-                            mesh_mapping[0] not in {'EFO', 'HP', 'DOID'}) else \
-                        ('MESH', mesh_id, mesh_name)
-        # Next we look at mappings to DOID
-        # TODO: are we sure that the DOIDs that we get here (from e.g., EFO)
-        # cannot be mapped further to MeSH per the DOID resource file?
-        elif 'DOID' in xref_dict:
-            doid = xref_dict['DOID']
-            if not doid.startswith('DOID:'):
-                doid = 'DOID:' + doid
-            doid_prim_id = doid_client.get_doid_id_from_doid_alt_id(doid)
-            if doid_prim_id:
-                doid = doid_prim_id
-            doid_name = doid_client.get_doid_name_from_doid_id(doid)
-            db, db_id, name = 'DOID', doid, doid_name
-
-        # Add a term for the name first
-        name_term = Term(
-            norm_text=normalize(name),
-            text=name,
-            db=db,
-            id=db_id,
-            entry_name=name,
-            status='name',
-            source=prefix,
-        )
-        terms.append(name_term)
-
-        # Then add all the synonyms
-        for synonym in set(entry['synonyms']):
-            # Some synonyms are tagged as ambiguous, we remove these
-            if 'ambiguous' in synonym.lower():
-                continue
-            # Some synonyms contain a "formerly" clause, we remove these
-            match = re.match(r'(.+) \(formerly', synonym)
-            if match:
-                synonym = match.groups()[0]
-            # Some synonyms contain additional annotations
-            # e.g. Hyperplasia of facial adipose tissue" NARROW
-            # [ORCID:0000-0001-5889-4463]
-            # If this is the case, we strip these off
-            match = re.match(r'([^"]+)', synonym)
-            if match:
-                synonym = match.groups()[0]
-
-            synonym_term = Term(
-                norm_text=normalize(synonym),
-                text=synonym,
-                db=db,
-                id=db_id,
-                entry_name=name,
-                status='synonym',
-                source=prefix,
-            )
-            terms.append(synonym_term)
-
+        terms += terms_from_obo_json_entry(entry, prefix=prefix,
+                                           ignore_mappings=ignore_mappings,
+                                           map_to_ns=map_to_ns)
     logger.info('Loaded %d terms from %s', len(terms), prefix)
     return terms
 
@@ -498,6 +510,20 @@ def filter_out_duplicates(terms):
     return new_terms
 
 
+def terms_from_obo_url(url, prefix, ignore_mappings=False, map_to_ns=None):
+    """Return terms extracted directly from an OBO given as a URL."""
+    import obonet
+    from indra.databases.obo_client import OboClient
+    g = obonet.read_obo(url)
+    entries = OboClient.entries_from_graph(g, prefix=prefix)
+    terms = []
+    for entry in entries:
+        terms += terms_from_obo_json_entry(entry, prefix=prefix,
+                                           ignore_mappings=ignore_mappings,
+                                           map_to_ns=map_to_ns)
+    return terms
+
+
 def get_all_terms():
     terms = []
 
